@@ -18,17 +18,347 @@
 */
 
 #include "codemodel.h"
+#include "driver.h"
 #include <kdebug.h>
+#include <kdatastream.h>
+
+///Little helper-functions to save a lot of typing and possible errors
+template<class MapContainer>
+bool eachCanUpdate( const MapContainer& old, const MapContainer& newMap ) {
+  if( old.size() != newMap.size() ) return false;
+
+  typename MapContainer::const_iterator oldIt = old.begin();
+  typename MapContainer::const_iterator newIt = newMap.begin();
+  while( oldIt != old.end() ) {
+    typedef typename MapContainer::mapped_type ListType;
+    if( (*oldIt).size() != (*newIt).size() ) return false;
+    typename ListType::const_iterator it1 = (*oldIt).begin();
+    typename ListType::const_iterator it2 = (*newIt).begin();
+
+    while( it1 != (*oldIt).end() ) {
+      if( !(*it1)->canUpdate( *it2 ) ) return false;
+      ++it1;
+      ++it2;
+    }
+    ++oldIt;
+    ++newIt;
+  }
+  return true;
+}
+
+template<class MapContainer>
+void eachUpdate( MapContainer& old, const MapContainer& newMap  ) {
+  if( old.size() != newMap.size() ) kdError( 9007 ) << "error in eachUpdate(...) 1" << endl;
+  typename MapContainer::iterator oldIt = old.begin();
+  typename MapContainer::const_iterator newIt = newMap.begin();
+  while( oldIt != old.end() ) {
+    if( (*oldIt).size() != (*newIt).size() ) kdError( 9007 ) << "error in eachUpdate(...) 2" << endl;
+    typedef typename MapContainer::mapped_type ListType;
+    typename ListType::iterator it1 = (*oldIt).begin();
+    typename ListType::const_iterator it2 = (*newIt).begin();
+    while( it1 != (*oldIt).end() ) {
+      (*it1)->update( *it2 );
+      ++it1;
+      ++it2;
+    }
+    ++oldIt;
+    ++newIt;
+  }
+}
+
+///Versions for contains that do not contain maps again
+template<class MapContainer>
+bool eachCanUpdateSingle( const MapContainer& old, const MapContainer& newMap ) {
+  if( old.size() != newMap.size() ) return false;
+
+  typename MapContainer::const_iterator oldIt = old.begin();
+  typename MapContainer::const_iterator newIt = newMap.begin();
+  while( oldIt != old.end() ) {
+    if( !(*oldIt)->canUpdate( *newIt ) ) return false;
+    ++oldIt;
+    ++newIt;
+  }
+  return true;
+}
+
+template<class MapContainer>
+void eachUpdateSingle( MapContainer& old, const MapContainer& newMap  ) {
+  if( old.size() != newMap.size() ) kdError( 9007 ) << "error in eachUpdate(...) 1" << endl;
+  typename MapContainer::iterator oldIt = old.begin();
+  typename MapContainer::const_iterator newIt = newMap.begin();
+  while( oldIt != old.end() ) {
+    (*oldIt)->update( *oldIt );
+    ++oldIt;
+    ++newIt;
+  }
+}
 
 CodeModel::CodeModel()
 {
     wipeout();
+    m_currentGroupId = 1;   ///0 stands for invalid group
 }
 
 CodeModel::~ CodeModel( )
 {
 }
 
+int CodeModel::newGroupId() {
+    return (m_currentGroupId++) * 2;
+}
+
+inline bool isSingleGroup( const int group ) {
+    return (group % 2) == 0;
+}
+
+QStringList CodeModel::getGroupStrings(int gid) const {
+    QStringList ret;
+    for(QMap<QString, FileDom>::ConstIterator it = m_files.begin(); it != m_files.end(); ++it) {
+        if((*it)->groupId() == gid) ret.append( (*it)-> name() );
+    }
+    return ret;
+}
+
+
+FileList CodeModel::getGroup(int gid) const {
+    FileList ret;
+    for(QMap<QString, FileDom>::ConstIterator it = m_files.begin(); it != m_files.end(); ++it) {
+        if((*it)->groupId() == gid) ret.append(*it);
+    }
+    return ret;
+}
+
+FileList CodeModel::getGroup( const FileDom& dom) const {
+    return getGroup( dom->groupId() );
+}
+
+int CodeModel::mergeGroups( int g1, int g2) {
+    if( !g1 || !g2 ) return 0;
+    if( g1 == g2 ) return g1;
+    int ng = isSingleGroup( g1 ) ? g2 : g1;
+    if( isSingleGroup( ng ) )
+        ng = newGroupId() + 1;
+    
+    for( QMap<QString, FileDom>::iterator it = m_files.begin(); it != m_files.end(); ++it ) {
+        if( (*it)->groupId() == g2 || (*it)->groupId() == g1 ) (*it)->setGroupId( ng );
+    }
+    return ng;
+}
+
+template<class Type> static void dumpMap( std::ostream& file, QMap<QString, Type>& map ) {
+    typename QMap<QString, Type>::Iterator it = map.begin();
+    for( ; it != map.end(); ++it) {
+        typename Type::Iterator it2 = (*it).begin();
+        for( ; it2 != (*it).end(); ++it2) {
+            (*it2) -> dump( file, true );
+        }
+    }
+}
+
+template<class Type> static void dumpMapDirect( std::ostream& file, QMap<QString, Type>& map ) {
+    typename QMap<QString, Type>::Iterator it = map.begin();
+    for( ; it != map.end(); ++it) {
+        (*it) -> dump( file, true );
+    }
+}
+
+void CodeModelItem::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+    
+    str << "name: " << name().ascii() << "\n";
+    str << "kind: " << m_kind << "  ";
+    
+    if( isFile() ) str << "isFile ";
+    if( isNamespace() ) str << "isNamespace ";
+    if( isClass() ) str << "isClass ";
+    if( isFunction() ) str << "isFunction ";
+    if( isFunctionDefinition() ) str << "isFunctionDefinition ";
+    if( isVariable() ) str << "isVariable ";
+    if( isArgument() ) str << "isArgument ";
+    if( isEnum() ) str << "isEnum ";
+    if( isEnumerator() ) str << "isEnumerator ";
+    if( isTypeAlias() ) str << "isTypeAlias ";
+    if( isCustom() ) str << "isCustom ";
+    str << "\n";
+    str << "File: " << fileName().ascii() << " ";
+    int line, col;
+    getStartPosition( &line, &col );
+    str << "s:(" << line << ", " << col << ") ";
+    getEndPosition( &line, &col );
+    str << "e:(" << line << ", " << col << ")\n";
+    
+    
+    Info.prepend( str.str().c_str() );
+    
+    file << Info.ascii() << "\n";
+    if(recurse) {} ///just to get rid of the warning
+}
+
+void ClassModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+    
+    
+    str << "scope: " << m_scope.join("::").ascii() << "\n";
+    str << "bases: " << m_baseClassList.join(" ").ascii() << "\n";
+    
+    Info.prepend( str.str().c_str() );
+    
+    CodeModelItem::dump( file, false, Info );
+    
+    if( recurse ) {
+        dumpMap( file, m_classes );
+    }
+}
+
+void NamespaceAliasModel::read( QDataStream& stream ) {
+  QString tempFileName;
+  stream >> m_name >> m_aliasName >> tempFileName;
+  m_fileName = HashedString( tempFileName );
+}
+
+void NamespaceAliasModel::write( QDataStream& stream ) const {
+  stream << m_name << m_aliasName << m_fileName.str();
+}
+
+void NamespaceImportModel::read( QDataStream& stream ) {
+  QString tempFileName;
+  stream >> m_name >> tempFileName;
+  m_fileName = HashedString( tempFileName );
+}
+
+void NamespaceImportModel::write( QDataStream& stream ) const {
+  stream << m_name << m_fileName.str();
+}
+
+void NamespaceModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    Info.prepend( str.str().c_str() );
+
+    ClassModel::dump( file, false, Info );
+
+    if( recurse ) {
+        dumpMapDirect( file, m_namespaces );
+    }
+}    
+
+void ArgumentModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    str << "type: " << m_type.ascii() << " default: " << m_defaultValue.ascii() << "\n";
+    
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+    
+    if(recurse) {} ///just to get rid of the warning
+}
+
+void FunctionModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    str << "access: " << m_access;
+        
+    str << " scope: " << m_scope.join("::").ascii() << "\n";
+        
+    if(isAbstract()) str << "isAbstract ";
+    if(isConstant()) str << "isConstant ";
+    if(isFunction()) str << "isFunction ";
+    if(isInline()) str << "isInline ";
+    if(isSignal()) str << "isSignal ";
+    if(isSlot()) str << "isSlot ";
+    if(isStatic()) str << "isStatic ";
+    if(isVirtual()) str << "isVirtual ";
+        
+    str << "\n";
+    str << "result-type: " << resultType().ascii() << "\n";
+        
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+
+    if(recurse) {
+        for( ArgumentList::iterator it = m_arguments.begin(); it != m_arguments.end(); ++it) {
+            (*it) -> dump( file, true );
+        }
+    }
+}
+
+void VariableModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    str << "access: " << m_access << "type: " << m_type.ascii() << "\n";
+    
+    if(isStatic()) str << "isStatic ";
+
+    str << "\n";
+
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+    
+    if(recurse) {} ///just to get rid of the warning
+}
+
+void CodeModel::dump( std::ostream& file, QString Info ) {
+    ostringstream str(ostringstream::out);
+            
+    Info.prepend( str.str().c_str() );
+    
+    file << Info.ascii() << "\n";
+    
+    QMap<QString, FileDom>::iterator it = m_files.begin();
+    for(; it != m_files.end(); ++it) {
+        (*it) -> dump( file, true );
+    }
+}
+
+void EnumModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    str << "access: " << m_access << "\n";
+    
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+    
+    if( recurse ) {
+        dumpMapDirect( file, m_enumerators );
+    }
+}
+
+void EnumeratorModel::dump( std::ostream& file, bool recurse, QString Info ) 
+{
+    ostringstream str( ostringstream::out );
+
+    str << "value: " << m_value.ascii() << "\n";
+    
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+    
+    if(recurse) {} ///just to get rid of the warning
+}
+
+void TypeAliasModel::dump( std::ostream& file, bool recurse, QString Info ) {
+    ostringstream str( ostringstream::out );
+
+    str << "type: " << m_type.ascii() << "\n";
+    
+    Info.prepend( str.str().c_str() );
+
+    CodeModelItem::dump( file, false, Info );
+    
+    if(recurse) {} ///just to get rid of the warning
+}
+
 void CodeModel::wipeout()
 {
     m_files.clear();
@@ -55,12 +385,22 @@ bool CodeModel::hasFile( const QString & name ) const
 
 FileDom CodeModel::fileByName( const QString & name )
 {
-    return m_files.contains(name) ? m_files[ name ] : FileDom();
+    QMap<QString, FileDom>::const_iterator it = m_files.find( name );
+    if( it != m_files.end() ) {
+      return *it;
+    } else {
+      return FileDom();
+    }
 }
 
 const FileDom CodeModel::fileByName( const QString & name ) const
 {
-    return m_files.contains(name) ? m_files[ name ] : FileDom();
+    QMap<QString, FileDom>::const_iterator it = m_files.find( name );
+    if( it != m_files.end() ) {
+      return *it;
+    } else {
+      return FileDom();
+    }
 }
 
 void CodeModel::addNamespace( NamespaceDom target, NamespaceDom source )
@@ -84,6 +424,8 @@ void CodeModel::addNamespace( NamespaceDom target, NamespaceDom source )
     VariableList variableList = source->variableList();
     EnumList enumList = source->enumList();
     TypeAliasList typeAliasList = source->typeAliasList();
+    const NamespaceModel::NamespaceAliasModelList& namespaceAliases = source->namespaceAliases();
+    const NamespaceModel::NamespaceImportModelList& namespaceImports = source->namespaceImports();
 
     for( NamespaceList::Iterator it=namespaceList.begin(); it!=namespaceList.end(); ++it )
 	addNamespace( ns, *it );
@@ -99,6 +441,10 @@ void CodeModel::addNamespace( NamespaceDom target, NamespaceDom source )
 	ns->addEnum( *it );
     for( TypeAliasList::Iterator it=typeAliasList.begin(); it!=typeAliasList.end(); ++it )
 	ns->addTypeAlias( *it );
+  for( NamespaceModel::NamespaceAliasModelList::const_iterator it=namespaceAliases.begin(); it != namespaceAliases.end(); ++it )
+    ns->addNamespaceAlias( *it );
+  for( NamespaceModel::NamespaceImportModelList::const_iterator it=namespaceImports.begin(); it != namespaceImports.end(); ++it )
+    ns->addNamespaceImport( *it );
 }
 
 void CodeModel::removeNamespace( NamespaceDom target, NamespaceDom source )
@@ -115,7 +461,9 @@ void CodeModel::removeNamespace( NamespaceDom target, NamespaceDom source )
     VariableList variableList = source->variableList();
     EnumList enumList = source->enumList();
     TypeAliasList typeAliasList = source->typeAliasList();
-
+    const NamespaceModel::NamespaceAliasModelList& namespaceAliases = source->namespaceAliases();
+    const NamespaceModel::NamespaceImportModelList& namespaceImports = source->namespaceImports();
+  
     for( NamespaceList::Iterator it=namespaceList.begin(); it!=namespaceList.end(); ++it )
 	removeNamespace( ns, *it );
     for( ClassList::Iterator it=classList.begin(); it!=classList.end(); ++it )
@@ -130,14 +478,20 @@ void CodeModel::removeNamespace( NamespaceDom target, NamespaceDom source )
 	ns->removeEnum( *it );
     for( TypeAliasList::Iterator it=typeAliasList.begin(); it!=typeAliasList.end(); ++it )
 	ns->removeTypeAlias( *it );
-
+  for( NamespaceModel::NamespaceAliasModelList::const_iterator it=namespaceAliases.begin(); it != namespaceAliases.end(); ++it )
+    ns->removeNamespaceAlias( *it );
+  for( NamespaceModel::NamespaceImportModelList::const_iterator it=namespaceImports.begin(); it != namespaceImports.end(); ++it )
+    ns->removeNamespaceImport( *it );
+  
     if( ns->namespaceList().isEmpty() && 
     	ns->classList().isEmpty() && 
 	ns->functionList().isEmpty() && 
 	ns->functionDefinitionList().isEmpty() && 
 	ns->variableList().isEmpty() &&
 	ns->enumList().isEmpty() && 
-	ns->typeAliasList().isEmpty() )
+	ns->typeAliasList().isEmpty() &&
+  ns->namespaceImports().empty() &&
+  ns->namespaceAliases().empty() )
     {
         target->removeNamespace( ns );
     }
@@ -146,7 +500,13 @@ void CodeModel::removeNamespace( NamespaceDom target, NamespaceDom source )
 bool CodeModel::addFile( FileDom file )
 {
     if( file->name().isEmpty() )
-	return false;
+		return false;
+
+		if( m_files.find( file->name() ) != m_files.end() ) {
+			///the error-channel is set to 9007 because this problem appears with the cpp-support, so it is needed while debugging it
+			kdDebug(9007) << "file " << file->name() << " was added to code-model without removing it before! \n" << kdBacktrace() << endl;
+			removeFile( fileByName( file->name() ) );
+		}
 
     // update global namespace
     NamespaceList namespaceList = file->namespaceList();
@@ -156,7 +516,9 @@ bool CodeModel::addFile( FileDom file )
     VariableList variableList = file->variableList();
     EnumList enumList = file->enumList();
     TypeAliasList typeAliasList = file->typeAliasList();
-
+    const NamespaceModel::NamespaceAliasModelList& namespaceAliases = file->namespaceAliases();
+    const NamespaceModel::NamespaceImportModelList& namespaceImports = file->namespaceImports();
+  
     for( NamespaceList::Iterator it=namespaceList.begin(); it!=namespaceList.end(); ++it )
 	addNamespace( m_globalNamespace, *it );
     for( ClassList::Iterator it=classList.begin(); it!=classList.end(); ++it )
@@ -171,7 +533,11 @@ bool CodeModel::addFile( FileDom file )
 	m_globalNamespace->addEnum( *it );
     for( TypeAliasList::Iterator it=typeAliasList.begin(); it!=typeAliasList.end(); ++it )
 	m_globalNamespace->addTypeAlias( *it );
-
+  for( NamespaceModel::NamespaceAliasModelList::const_iterator it=namespaceAliases.begin(); it != namespaceAliases.end(); ++it )
+    m_globalNamespace->addNamespaceAlias( *it );
+  for( NamespaceModel::NamespaceImportModelList::const_iterator it=namespaceImports.begin(); it != namespaceImports.end(); ++it )
+    m_globalNamespace->addNamespaceImport( *it );
+  
     m_files.insert( file->name(), file );
     return true;
 }
@@ -186,7 +552,9 @@ void CodeModel::removeFile( FileDom file )
     VariableList variableList = file->variableList();
     EnumList enumList = file->enumList();
     TypeAliasList typeAliasList = file->typeAliasList();
-
+    const NamespaceModel::NamespaceAliasModelList& namespaceAliases = file->namespaceAliases();
+    const NamespaceModel::NamespaceImportModelList& namespaceImports = file->namespaceImports();
+  
     for( NamespaceList::Iterator it=namespaceList.begin(); it!=namespaceList.end(); ++it )
 	removeNamespace( m_globalNamespace, *it );
     for( ClassList::Iterator it=classList.begin(); it!=classList.end(); ++it )
@@ -201,7 +569,11 @@ void CodeModel::removeFile( FileDom file )
 	m_globalNamespace->removeEnum( *it );
     for( TypeAliasList::Iterator it=typeAliasList.begin(); it!=typeAliasList.end(); ++it )
 	m_globalNamespace->removeTypeAlias( *it );
-	
+  for( NamespaceModel::NamespaceAliasModelList::const_iterator it=namespaceAliases.begin(); it != namespaceAliases.end(); ++it )
+    m_globalNamespace->removeNamespaceAlias( *it );
+  for( NamespaceModel::NamespaceImportModelList::const_iterator it=namespaceImports.begin(); it != namespaceImports.end(); ++it )
+    m_globalNamespace->removeNamespaceImport( *it );
+  
     m_files.remove( file->name() );
 }
 
@@ -274,6 +646,20 @@ void CodeModelItem::setEndPosition( int line, int column )
     m_endColumn = column;
 }
 
+void CodeModelItem::update( const CodeModelItem* i ) {
+  m_startLine = i->m_startLine;
+  m_startColumn = i->m_startColumn;
+  m_endLine = i->m_endLine;
+  m_endColumn = i->m_endColumn;
+}
+
+bool CodeModelItem::canUpdate( const CodeModelItem* i ) const {
+  if( i->m_kind != m_kind || i->m_name != m_name )
+    return false;
+  return true;
+}
+
+
 // ------------------------------------------------------------------------
 NamespaceModel::NamespaceModel( CodeModel* model )
     : ClassModel( model )
@@ -322,7 +708,7 @@ void NamespaceModel::removeNamespace( NamespaceDom ns )
 
 // ------------------------------------------------------------------------
 FileModel::FileModel( CodeModel* model )
-    : NamespaceModel( model)
+    : NamespaceModel( model ), m_groupId( model->newGroupId() ), m_parseResult( 0 )
 {
 }
 
@@ -589,6 +975,28 @@ bool ClassModel::addEnum( EnumDom e )
     return true;
 }
 
+void ClassModel::update( const ClassModel* klass ) {
+  CodeModelItem::update( klass );
+  eachUpdate( m_classes, klass->m_classes ) ;
+  eachUpdate( m_functions, klass->m_functions ) ;
+  eachUpdate( m_functionDefinitions, klass->m_functionDefinitions ) ;
+  eachUpdateSingle( m_variables, klass->m_variables ) ;
+  eachUpdateSingle( m_enumerators, klass->m_enumerators ) ;
+  eachUpdate( m_typeAliases, klass->m_typeAliases );
+}
+
+bool ClassModel::canUpdate( const ClassModel* klass ) const {
+    if( !CodeModelItem::canUpdate( klass ) )
+      return false;
+    
+    return eachCanUpdate( m_classes, klass->m_classes ) &&
+        eachCanUpdate( m_functions, klass->m_functions ) &&
+        eachCanUpdate( m_functionDefinitions, klass->m_functionDefinitions ) &&
+        eachCanUpdateSingle( m_variables, klass->m_variables ) &&
+        eachCanUpdateSingle( m_enumerators, klass->m_enumerators ) &&
+        eachCanUpdate( m_typeAliases, klass->m_typeAliases );
+}
+
 void ClassModel::removeEnum( EnumDom e )
 {
     m_enumerators.remove( e->name() );
@@ -773,12 +1181,26 @@ void FunctionModel::removeArgument( ArgumentDom arg )
     m_arguments.remove( arg );
 }
 
+void FunctionModel::update( const FunctionModel* i ) {
+  CodeModelItem::update( i );
+}
+
+bool FunctionModel::canUpdate( const FunctionModel* i ) const {
+  if( !CodeModelItem::canUpdate( i ) )
+    return false;
+  if( m_resultType != i->m_resultType || m_arguments.count() != i->m_arguments.count() || m_scope != i->m_scope )
+    return false;
+  return true;
+}
+
+
 // ------------------------------------------------------------------------
 VariableModel::VariableModel( CodeModel* model )
     : CodeModelItem( Variable, model)
 {
     m_access = Public;
     m_static = false;
+		m_isEnumeratorVariable = false;
 }
 
 bool VariableModel::isStatic( ) const
@@ -801,6 +1223,14 @@ void VariableModel::setType( const QString& type )
     m_type = type;
 }
 
+bool VariableModel::isEnumeratorVariable() const {
+	return m_isEnumeratorVariable;
+}
+
+void VariableModel::setEnumeratorVariable( bool b) {
+	m_isEnumeratorVariable = b;
+}
+
 int FunctionModel::access( ) const
 {
     return m_access;
@@ -860,7 +1290,14 @@ void CodeModelItem::read( QDataStream & stream )
 	>> m_startLine
 	>> m_startColumn
 	>> m_endLine
-	>> m_endColumn;
+	>> m_endColumn
+        >> m_comment;
+    
+    if( isTemplateable() ) {
+        TemplateModelItem* t = (TemplateModelItem*)( this );
+        
+        t->read( stream );
+    }
 }
 
 void CodeModelItem::write( QDataStream & stream ) const
@@ -872,13 +1309,21 @@ void CodeModelItem::write( QDataStream & stream ) const
 	<< m_startLine
 	<< m_startColumn
 	<< m_endLine
-	<< m_endColumn;
+	<< m_endColumn
+        << m_comment;
+    
+    if( isTemplateable() ) {
+        TemplateModelItem* t = (TemplateModelItem*)( this );
+        t-> write( stream );
+    }
 }
 
 void ClassModel::read( QDataStream & stream )
 {
     CodeModelItem::read( stream );
 
+		TemplateModelItem::read( stream );
+		
     stream >> m_scope >> m_baseClassList;
 
     int n;
@@ -936,6 +1381,8 @@ void ClassModel::write( QDataStream & stream ) const
 {
     CodeModelItem::write( stream );
 
+		TemplateModelItem::write( stream );
+		
     stream << m_scope << m_baseClassList;
 
     const ClassList class_list = classList();
@@ -976,13 +1423,42 @@ void NamespaceModel::read( QDataStream & stream )
 
     int n;
 
-    m_namespaces.clear();
+    m_namespaces.clear(); m_namespaceAliases.clear(); m_namespaceImports.clear();
     stream >> n;
     for( int i=0; i<n; ++i ){
 	NamespaceDom ns = codeModel()->create<NamespaceModel>();
 	ns->read( stream );
 	addNamespace( ns );
     }
+
+  stream >> n;
+  for( int a = 0; a < n; a++ ) {
+    NamespaceAliasModel m;
+    m.read( stream );
+    m_namespaceAliases.insert( m );
+  }
+  stream >> n;
+  for( int a = 0; a < n; a++ ) {
+    NamespaceImportModel m;
+    m.read( stream );
+    m_namespaceImports.insert( m );
+  }
+}
+
+void NamespaceModel::addNamespaceImport( const NamespaceImportModel& import ) {
+  m_namespaceImports.insert( import );
+}
+
+void NamespaceModel::addNamespaceAlias( const NamespaceAliasModel& alias ) {
+  m_namespaceAliases.insert( alias );
+}
+
+void NamespaceModel::removeNamespaceImport( const NamespaceImportModel& import ) {
+  m_namespaceImports.erase( import );
+}
+
+void NamespaceModel::removeNamespaceAlias( const NamespaceAliasModel& alias ) {
+  m_namespaceAliases.erase( alias );
 }
 
 void NamespaceModel::write( QDataStream & stream ) const
@@ -993,21 +1469,72 @@ void NamespaceModel::write( QDataStream & stream ) const
     stream << int( namespace_list.size() );
     for( NamespaceList::ConstIterator it = namespace_list.begin(); it!=namespace_list.end(); ++it )
 	(*it)->write( stream );
+
+  stream << int( m_namespaceAliases.size() );
+  for( NamespaceAliasModelList::const_iterator it = m_namespaceAliases.begin(); it != m_namespaceAliases.end(); ++it )
+    (*it).write( stream );
+  stream << int( m_namespaceImports.size() );
+  for( NamespaceImportModelList::const_iterator it = m_namespaceImports.begin(); it != m_namespaceImports.end(); ++it )
+    (*it).write( stream );
+}
+
+bool NamespaceModel::canUpdate( const NamespaceModel* ns ) const {
+  if( !ClassModel::canUpdate( ns ) )
+    return false;
+    ///@todo check namespace-aliases and imports
+//     const NamespaceModel::NamespaceAliasModelList& namespaceAliases = file->namespaceAliases();
+//     const NamespaceModel::NamespaceImportModelList& namespaceImports = file->namespaceImports();
+
+    return eachCanUpdateSingle( m_namespaces, ns->m_namespaces );
+}
+
+void NamespaceModel::update( const NamespaceModel* ns )
+{
+  ClassModel::update( ns );
+    ///@todo update namespace-aliases and imports
+//     const NamespaceModel::NamespaceAliasModelList& namespaceAliases = file->namespaceAliases();
+//     const NamespaceModel::NamespaceImportModelList& namespaceImports = file->namespaceImports();
+
+    eachUpdateSingle( m_namespaces, ns->m_namespaces );
 }
 
 void FileModel::read( QDataStream & stream )
 {
+    stream >> m_groupId;
+    bool b;
+    stream >> b;
+    if( b ) {
+      int i;
+      stream >> i;
+      ParsedFileType t( (ParsedFileType) i );
+      switch( t ) {
+      case CppParsedFile:
+        m_parseResult = (AbstractParseResult*)(new ParsedFile( stream ));
+        break;
+      }
+    }
+
     NamespaceModel::read( stream );
 }
 
 void FileModel::write( QDataStream & stream ) const
 {
+    stream << m_groupId;
+    bool b = m_parseResult;
+    stream << b;
+    if( b ) {
+        int i = m_parseResult->type();
+        stream << i;
+        m_parseResult->write( stream );
+    }
+    
     NamespaceModel::write( stream );
 }
 
 void ArgumentModel::read( QDataStream & stream )
 {
     CodeModelItem::read( stream );
+  
 
     stream >> m_type >> m_defaultValue;
 }
@@ -1022,6 +1549,7 @@ void ArgumentModel::write( QDataStream & stream ) const
 void FunctionModel::read( QDataStream & stream )
 {
     CodeModelItem::read( stream );
+		TemplateModelItem::read( stream );
 
     stream >> m_scope;
     stream >> d.flags;
@@ -1043,6 +1571,7 @@ void FunctionModel::read( QDataStream & stream )
 void FunctionModel::write( QDataStream & stream ) const
 {
     CodeModelItem::write( stream );
+		TemplateModelItem::write( stream );
 
     stream << m_scope;
     stream << d.flags;
@@ -1081,13 +1610,25 @@ void CodeModel::write( QDataStream & stream ) const
 void VariableModel::read( QDataStream & stream )
 {
     CodeModelItem::read( stream );
-    stream >> m_access >> m_static >> m_type;
+		stream >> m_access >> m_static >> m_type >> m_isEnumeratorVariable;
 }
 
 void VariableModel::write( QDataStream & stream ) const
 {
     CodeModelItem::write( stream );
-    stream << m_access << m_static << m_type;
+		stream << m_access << m_static << m_type << m_isEnumeratorVariable;
+}
+
+void VariableModel::update( const VariableModel* i ) {
+  CodeModelItem::update( i );
+}
+
+bool VariableModel::canUpdate( const VariableModel* i ) const {
+  if( !CodeModelItem::canUpdate( i ) )
+    return false;
+  if( m_access != i->m_access || m_static != i->m_static || m_type != i->m_type || m_isEnumeratorVariable != i->m_isEnumeratorVariable )
+    return false;
+  return true;
 }
 
 // -------------------------------------------------------
@@ -1178,6 +1719,19 @@ void EnumModel::removeEnumerator( EnumeratorDom e )
     m_enumerators.remove( e->name() );
 }
 
+void EnumModel::update( const EnumModel* i ) {
+  CodeModelItem::update( i );
+}
+
+bool EnumModel::canUpdate( const EnumModel* i ) const {
+  if( !CodeModelItem::canUpdate( i ) )
+    return false;
+  ///@todo check not complete
+  if( m_access != i->m_access || m_enumerators.count() != i->m_enumerators.count() )
+    return false;
+  return true;
+}
+
 // ---------------------------------------------------------------
 TypeAliasModel::TypeAliasModel( CodeModel * model )
     : CodeModelItem( TypeAlias, model )
@@ -1208,3 +1762,30 @@ void TypeAliasModel::setType( const QString & type )
     m_type = type;
 }
 
+void TypeAliasModel::update( const TypeAliasModel* i ) {
+  CodeModelItem::update( i );
+}
+
+bool TypeAliasModel::canUpdate( const TypeAliasModel* i ) const {
+  if( !CodeModelItem::canUpdate( i ) )
+    return false;
+  return m_type == i->m_type;
+}
+
+FileList FileModel::wholeGroup() {
+    if( isSingleGroup( m_groupId ) ) return ( FileList() << FileDom(this) );
+    return codeModel()->getGroup( m_groupId );
+}
+
+QStringList FileModel::wholeGroupStrings() const {
+    if( isSingleGroup( m_groupId ) ) return (QStringList() << name() );
+    return codeModel()->getGroupStrings( m_groupId );
+}
+
+ParseResultPointer FileModel::parseResult() const {
+    return m_parseResult;
+}
+
+void FileModel::setParseResult( const ParseResultPointer& result ) {
+    m_parseResult = result;
+}
