@@ -32,6 +32,7 @@
 #include <qtoolbutton.h>
 #include <qtooltip.h>
 #include <qvalidator.h>
+#include <qtimer.h>
 #include <klistview.h>
 #include <kiconview.h>
 #include <kconfig.h>
@@ -66,6 +67,7 @@
 #include "kdevversioncontrol.h"
 #include "kdevmakefrontend.h"
 #include "kdevpartcontroller.h"
+#include "kdevplugincontroller.h"
 #include "kdevappfrontend.h"
 #include "kdevplugininfo.h"
 #include "kdevlicense.h"
@@ -76,6 +78,10 @@
 #include "misc.h"
 #include "profilesupport.h"
 #include "filetemplate.h"
+#include "settings.h"
+
+#include "profileengine.h"
+#include "profile.h"
 
 #include "propeditor/property.h"
 #include "propeditor/multiproperty.h"
@@ -83,7 +89,7 @@
 #include "propeditor/propertyeditor.h"
 
 AppWizardDialog::AppWizardDialog(AppWizardPart *part, QWidget *parent, const char *name)
-    : AppWizardDialogBase(parent, name,true), m_pCurrentAppInfo(0), 
+    : AppWizardDialogBase(parent, name,true), m_pCurrentAppInfo(0),
 	m_profileSupport(new ProfileSupport(part))
 {
 	kdDebug( 9000 ) << "  ** AppWizardDialog::AppWizardDialog()" << endl;
@@ -201,7 +207,7 @@ AppWizardDialog::AppWizardDialog(AppWizardPart *part, QWidget *parent, const cha
 				value.cast( variantType );  // fix this in kdelibs...
 				if( !name.isEmpty() && !label.isEmpty() )
 					info->propValues->addProperty( new PropertyLib::Property( (int)variantType, name, label, value ) );
-				
+
 			}
 			else if( type == "install" ) // copy dir
 			{
@@ -312,10 +318,10 @@ AppWizardDialog::AppWizardDialog(AppWizardPart *part, QWidget *parent, const cha
     }
 
 	connect( license_combo, SIGNAL(activated(int)), this, SLOT(licenseChanged()) );
-	
+
 	m_custom_options_layout = new QHBoxLayout( custom_options );
 	m_custom_options_layout->setAutoAdd(true);
-	
+
 	showTemplates(false);
 }
 
@@ -344,7 +350,7 @@ void AppWizardDialog::loadVcs()
 			QString errorMessage = KLibLoader::self()->lastErrorMessage();
 			kdDebug(9010) << "There was an error loading the module " << service->name() << endl <<
 			"The diagnostics is:" << endl << errorMessage << endl;
-			exit(1);
+			continue;
 		}
 		QStringList args;
 		QObject *obj = factory->create(0, service->name().latin1(),
@@ -422,6 +428,8 @@ void AppWizardDialog::licenseChanged()
 				commentStyle = KDevFile::AdaStyle;
 			} else if (style == "ShellStyle") {
 				commentStyle = KDevFile::BashStyle;
+			} else if (style == "XMLStyle") {
+				commentStyle = KDevFile::XMLStyle;
 			}
 
 			QString text;
@@ -478,7 +486,7 @@ void AppWizardDialog::accept()
 	PropertyLib::PropertyList::Iterator idx = m_pCurrentAppInfo->propValues->begin();
 	for( ; idx != m_pCurrentAppInfo->propValues->end(); ++idx)
 		m_pCurrentAppInfo->subMap.insert( idx.data()->name(), idx.data()->value().toString() );
-	
+
 	m_pCurrentAppInfo->subMap.insert("src", archDir.name() );
 	m_pCurrentAppInfo->subMap.insert("dest", finalLoc_label->text() );
 	m_pCurrentAppInfo->subMap.insert("APPNAME", appname_edit->text() );
@@ -490,7 +498,7 @@ void AppWizardDialog::accept()
 	m_pCurrentAppInfo->subMap.insert("VERSION", version_edit->text());
 	m_pCurrentAppInfo->subMap.insert( "I18N", "i18n" );
 	m_pCurrentAppInfo->subMap.insert("YEAR", QString::number( QDate::currentDate().year() ) );
-	
+
 	// This isn't too pretty, but we have several templates that use KAboutData::License_${LICENSE}
 	// and unsurprisingly, KAboutData doesn't cover every imaginable case.
 	// These are the licenses known to KDE-3.2 KAboutData, KDevelop doesn't have all of these as prepared options today
@@ -522,8 +530,8 @@ void AppWizardDialog::accept()
 	QString tempProjectDomSource = "<!DOCTYPE kdevelop><kdevelop><general><author>%1</author><email>%2</email><version>%3</version></general></kdevelop>";
 	tempProjectDomSource = tempProjectDomSource.arg( author_edit->text() ).arg( email_edit->text() ).arg( version_edit->text() );
 	QDomDocument tempProjectDom;
-	tempProjectDom.setContent( tempProjectDomSource ); 
-	
+	tempProjectDom.setContent( tempProjectDomSource );
+
     QValueList<AppWizardFileTemplate>::Iterator it;
     for (it = m_fileTemplates.begin(); it != m_fileTemplates.end(); ++it) {
         KTempFile *tempFile = new KTempFile();
@@ -533,6 +541,7 @@ void AppWizardDialog::accept()
 		QFile f;
 		f.open(IO_WriteOnly, tempFile->handle());
 		QTextStream temps(&f);
+		temps.setEncoding(QTextStream::UnicodeUTF8);
 		temps << templateText;
 		f.flush();
 		QString templateName( QString( "%1_TEMPLATE" ).arg( (*it).suffix ).upper() );
@@ -675,7 +684,6 @@ void AppWizardDialog::accept()
            if (KDevAppFrontend *appFrontend = m_part->extension<KDevAppFrontend>("KDevelop/AppFrontend"))
               appFrontend->startAppCommand(KMacroExpander::expandMacros(m_pCurrentAppInfo->finishCmdDir, m_pCurrentAppInfo->subMap), KMacroExpander::expandMacros(m_pCurrentAppInfo->finishCmd, m_pCurrentAppInfo->subMap), false);
 
-	openAfterGeneration();
 
 	int id = m_vcsForm->stack->id(m_vcsForm->stack->visibleWidget());
 	if (id)
@@ -692,6 +700,7 @@ void AppWizardDialog::accept()
 	else
 		kdDebug(9010) << "vcs integrator wasn't selected" << endl;
 
+	openAfterGeneration();
 	QWizard::accept();
 }
 
@@ -701,9 +710,9 @@ bool AppWizardDialog::copyFile( const installFile& file )
 		copyFile( file.source, file.dest, file.isXML, file.process );
 }
 
-#include <sys/types.h> 
-#include <sys/stat.h> 
-#include <unistd.h> 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 bool AppWizardDialog::copyFile( const QString &source, const QString &dest, bool isXML, bool process )
 {
@@ -719,7 +728,9 @@ bool AppWizardDialog::copyFile( const QString &source, const QString &dest, bool
 		if( inputFile.open( IO_ReadOnly ) && outputFile.open(IO_WriteOnly) )
 		{
 			QTextStream input( &inputFile );
+			input.setEncoding(QTextStream::UnicodeUTF8);
 			QTextStream output( &outputFile );
+			output.setEncoding(QTextStream::UnicodeUTF8);
 			while( !input.atEnd() )
 				output << KMacroExpander::expandMacros(input.readLine(), subMap) << "\n";
 			// Preserve file mode...
@@ -821,7 +832,7 @@ void AppWizardDialog::templatesTreeViewClicked(QListViewItem *item)
 		// Populate new custom options form
 		m_customOptions = new PropertyLib::PropertyEditor( custom_options );
 		m_customOptions->populateProperties(info->propValues);
-		
+
 
         // Create new file template pages
         QStringList l = QStringList::split(",", info->fileTemplates);
@@ -872,7 +883,7 @@ void AppWizardDialog::destButtonClicked(const QString& dir)
         kdDebug(9010) << "DevPrjDir == newdir?: " << defPrjDir.absPath() << " == " << newDir.absPath() << endl;
         if (defPrjDir != newDir) {
             if (KMessageBox::questionYesNo(this, i18n("Set default project location to: %1?").arg( newDir.absPath() ),
-                                           i18n("New Project")) == KMessageBox::Yes)
+                                           i18n("New Project"), i18n("Set"), i18n("Do Not Set")) == KMessageBox::Yes)
             {
                 config->writePathEntry("DefaultProjectsDir", newDir.absPath() + "/");
                 config->sync();
@@ -892,9 +903,9 @@ void AppWizardDialog::projectLocationChanged()
 {
   // Jakob Simon-Gaarde: Got tired of the anoying bug with the appname/location confussion.
   // This version insures WYSIWYG and checks pathvalidity
-  finalLoc_label->setText(dest_edit->url() + (dest_edit->url().right(1)=="/" ? "":"/") + appname_edit->text().lower());
+  finalLoc_label->setText(dest_edit->url() + (dest_edit->url().right(1)=="/" ? "":"/") + appname_edit->text());
   QDir qd(dest_edit->url());
-  QFileInfo fi(dest_edit->url() + "/" + appname_edit->text().lower());
+  QFileInfo fi(dest_edit->url() + "/" + appname_edit->text());
   if (!qd.exists() || appname_edit->displayText().isEmpty()||fi.exists())
   {
     if (!fi.exists() || appname_edit->displayText().isEmpty()) {
@@ -958,7 +969,7 @@ void AppWizardDialog::openAfterGeneration()
 	if( !file.open( IO_ReadOnly ) )
 		return;
 	QDomDocument projectDOM;
-	
+
 	int errorLine, errorCol;
 	QString errorMsg;
 	bool success = projectDOM.setContent( &file, &errorMsg, &errorLine, &errorCol);
@@ -982,53 +993,47 @@ void AppWizardDialog::openAfterGeneration()
 		DomUtil::writeEntry(projectDOM, "/general/versioncontrol", service->property("X-KDevelop-VCSPlugin").toString());
 	}
 
-	
-	KConfig * config = kapp->config();
-	config->setGroup("IgnorePerDefault");
-	QStringList ignoreparts = config->readListEntry( "KDevelop" );
-	DomUtil::writeListEntry( projectDOM, "/general/ignoreparts", "part", ignoreparts );
-	
-	
-//FIXME PROFILES!!!!!!!!
-//BEGIN Plugin Profile
-
-/*	QString category = m_pCurrentAppInfo->category;
-	if ( category.left( 1 ) == "/" )
+	// figure out what plugins we should disable by default
+	QString profileName = DomUtil::readEntry( projectDOM, "general/profile" );
+	if ( profileName.isEmpty() )
 	{
-		category = category.right( category.length() -1 );
+		QString language = DomUtil::readEntry( projectDOM, "general/primarylanguage" );
+		QStringList keywords = DomUtil::readListEntry( projectDOM, "general/keywords", "keyword" );
+
+		profileName = Settings::profileByAttributes( language, keywords );
 	}
 
-	QString profile = Profiles::getProfileForCategory( category );
-	QStringList loadList = Profiles::getPluginsForProfile( profile );
+	ProfileEngine & engine = m_part->pluginController()->engine();
+	Profile * profile = engine.findProfile( profileName );
 
-	QStringList ignoreList;
-
-	KTrader::OfferList offers = KTrader::self()->query("KDevelop/Plugin", "[X-KDevelop-Scope] == 'Project'");
-	KTrader::OfferList::ConstIterator itt = offers.begin();
-	while( itt != offers.end() )
+	QStringList disableList;
+	Profile::EntryList disableEntryList = profile->list( Profile::ExplicitDisable );
+	for ( Profile::EntryList::const_iterator it = disableEntryList.constBegin(); it != disableEntryList.constEnd(); ++it )
 	{
-		if ( !loadList.contains( (*itt)->name() ) )
-		{
-			ignoreList << (*itt)->name();
-		}
-		++itt;
+		disableList << (*it).name;
 	}
 
-	DomUtil::writeListEntry( projectDOM, "/general/ignoreparts", "part", ignoreList );*/
+	QStringList projectIgnoreparts = DomUtil::readListEntry( projectDOM, "/general/ignoreparts", "part" );
+	projectIgnoreparts += disableList;
+	DomUtil::writeListEntry( projectDOM, "/general/ignoreparts", "part", projectIgnoreparts );
 
-//END Plugin Profile
+	DomUtil::writeEntry( projectDOM, "/general/projectname", appname_edit->text() );
 
 	// write the dom back
 	if( !file.open( IO_WriteOnly ) )
 		return;
 	QTextStream ts( &file );
+	ts.setEncoding(QTextStream::UnicodeUTF8);
 	ts << projectDOM.toString(2);
 	file.close();
 
 	// open the new project
 	m_part->core()->openProject( projectFile );
 
-	// open files to open
+	// calculate the list of files to open after generation and use
+	// timer to queue opening (so that files will not be opened before the project
+	// which is also queued by ProjectManager )
+	KURL::List urlsToOpen;
 	QStringList::Iterator it = m_pCurrentAppInfo->openFilesAfterGeneration.begin();
 	for( ; it != m_pCurrentAppInfo->openFilesAfterGeneration.end(); ++it )
 	{
@@ -1036,15 +1041,19 @@ void AppWizardDialog::openAfterGeneration()
 		if ( !fileName.isNull() )
 		{
 			fileName = KMacroExpander::expandMacros(fileName, m_pCurrentAppInfo->subMap);
-			m_part->partController()->editDocument( fileName );
+			urlsToOpen.append(KURL::fromPathOrURL(fileName));
 		}
 	}
+	m_part->openFilesAfterGeneration(urlsToOpen);
 }
 
 void AppWizardDialog::pageChanged()
 {
 	kdDebug(9010) << "AppWizardDialog::pageChanged()" << endl;
 	projectLocationChanged();
+	if (currentPage() == m_lastPage)
+		finishButton()->setDefault(true);
+
 
 	//it is possible that project name was changed - we need to update all vcs integrator dialogs
 	for (QMap<int, VCSDialog*>::iterator it = m_integratorDialogs.begin();
@@ -1264,13 +1273,13 @@ void AppWizardDialog::showTemplates(bool all)
 	else
 	{
 		QPtrListIterator<ApplicationInfo> ait(m_appsInfo);
-		for (; ait.current(); ++ait) 
+		for (; ait.current(); ++ait)
 		{
 			ait.current()->item->setVisible(m_profileSupport->isInTemplateList(ait.current()->templateName));
 		}
-		
+
 		QDictIterator<QListViewItem> dit(m_categoryMap);
-		for (; dit.current(); ++dit) 
+		for (; dit.current(); ++dit)
 		{
 			//checking whether all children are not visible
 			kdDebug(9010) << "check: " << dit.current()->text(0) << endl;
